@@ -10,7 +10,7 @@ Params::Validate::set_options( on_fail => sub { Alzabo::Exception::Params->throw
 
 use base qw(Alzabo::Column);
 
-$VERSION = sprintf '%2d.%02d', q$Revision: 1.33 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.34 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -103,7 +103,14 @@ sub set_name
     if ($@)
     {
 	$self->{name} = $old_name;
-	$@->rethrow;
+	if ( UNIVERSAL::can( $@, 'rethrow' ) )
+	{
+	    $@->rethrow;
+	}
+	else
+	{
+	    Alzabo::Exception->throw( error => $@ );
+	}
     }
 
     $self->table->register_column_name_change( column => $self,
