@@ -10,7 +10,7 @@ Params::Validate::set_options( on_fail => sub { Alzabo::Exception::Params->throw
 
 use base qw( Alzabo::Runtime::Cursor );
 
-$VERSION = sprintf '%2d.%02d', q$Revision: 1.13 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.15 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -45,7 +45,7 @@ sub next
     {
 	$self->{errors} = [];
 
-	my @data = $self->{statement}->next_row
+	my @data = $self->{statement}->next
 	    or return;
 
 	foreach my $t ( @{ $self->{tables} } )
@@ -74,7 +74,14 @@ sub next
 		}
 		else
 		{
-		    $@->rethrow;
+		    if ( UNIVERSAL::can( $@, 'rethrow' ) )
+		    {
+			$@->rethrow;
+		    }
+		    else
+		    {
+			Alzabo::Exception->throw( error => $@ );
+		    }
 		}
 	    }
 	    push @rows, $row;
